@@ -1,6 +1,6 @@
 --[[
 
-File l3build-check.lua Copyright (C) 2018 The LaTeX3 Project
+File l3build-check.lua Copyright (C) 2018,2019 The LaTeX3 Project
 
 It may be distributed and/or modified under the conditions of the
 LaTeX Project Public License (LPPL), either version 1.3c of this
@@ -98,27 +98,27 @@ end
 -- the 'business' part from the tests and removes system-dependent stuff
 local function normalize_log(content,engine,errlevels)
   local maxprintline = maxprintline
-  if engine == "luatex" or engine == "luajittex" then
+  if match(engine,"^lua") then
     maxprintline = maxprintline + 1 -- Deal with an out-by-one error
   end
   local function killcheck(line)
-      -- Skip lines containing file dates
-      if match(line, "[^<]%d%d%d%d/%d%d/%d%d") 
-         or match(line, "[^<]%d%d%d%d%-%d%d%-%d%d") then
-        return true
-      elseif
       -- Skip \openin/\openout lines in web2c 7.x
       -- As Lua doesn't allow "(in|out)", a slightly complex approach:
       -- do a substitution to check the line is exactly what is required!
-        match(
-          gsub(line, "^\\openin", "\\openout"), "^\\openout%d%d? = "
-        ) then
-        return true
-      end
+    if match(gsub(line, "^\\openin", "\\openout"), "^\\openout%d%d? = ") then
+      return true
+    end
     return false
   end
     -- Substitutions to remove some non-useful changes
-  local function normalize(line, lastline)
+  local function normalize(line,lastline,drop_fd)
+    if drop_fd then
+      if match(line," *%)") then
+        return "",""
+      else
+        return "","",true
+      end
+    end
     -- Zap line numbers from \show, \showbox, \box_show and the like:
     -- do this before wrapping lines
     line = gsub(line, "^l%.%d+ ", "l. ...")
@@ -151,12 +151,17 @@ local function normalize_log(content,engine,errlevels)
         line = gsub(line, pattern, "../%1")
       end
     end
-    -- Deal with the fact that "(.aux)" may have still a leading space
-    line = gsub(line, "^ %(%.aux%)", "(.aux)")
-    -- Merge all of .fd data into one line so will be removed later
+    -- Deal with dates
+    if match(line, "[^<]%d%d%d%d[/%-]%d%d[/%-]%d%d") then
+        line = gsub(line,"%d%d%d%d[/%-]%d%d[/%-]%d%d","....-..-..")
+        line = gsub(line,"v%d+%.?%d?%d?%w?","v...")
+    end
+    -- Deal with leading spaces for file and page number lines
+    line = gsub(line,"^ *%[(%d)","[%1")
+    line = gsub(line,"^ *%(","(")
+    -- Zap .fd lines: drop the first part, and skip to the end
     if match(line, "^ *%([%.%/%w]+%.fd[^%)]*$") then
-      lastline = (lastline or "") .. line
-      return "", (lastline or "") .. line
+      return "","",true
     end
     -- TeX90/XeTeX knows only the smaller set of dimension units
     line = gsub(line,
@@ -250,6 +255,7 @@ local function normalize_log(content,engine,errlevels)
     return line, lastline
   end
   local lastline = ""
+  local drop_fd = false
   local new_content = ""
   local prestart = true
   local skipping = false
@@ -264,7 +270,7 @@ local function normalize_log(content,engine,errlevels)
     elseif match(line, "^%)?TIMO$") then
       skipping = false
     elseif not prestart and not skipping then
-      line, lastline = normalize(line, lastline)
+      line, lastline, drop_fd = normalize(line, lastline,drop_fd)
       if not match(line, "^ *$") and not killcheck(line) then
         new_content = new_content .. line .. os_newline
       end
@@ -526,15 +532,18 @@ end
 -- Run one test which may have multiple engine-dependent comparisons
 -- Should create a difference file for each failed test
 function runcheck(name, hide)
-  if not testexists(name) then return 1 end
+  if not testexists(name) then
+    print("Failed to find input for test " .. name)
+    return 1
+  end
   local checkengines = checkengines
   if options["engine"] then
     checkengines = options["engine"]
   end
   -- Used for both .lvt and .pvt tests
-  local function check_and_diff(ext,engine,enginename,comp,pdftest)
+  local function check_and_diff(ext,engine,comp,pdftest)
     runtest(name,engine,hide,ext,pdftest,true)
-    local errorlevel = comp(name,enginename)
+    local errorlevel = comp(name,engine)
     if errorlevel ~= 0 and options["halt-on-error"] then
       showfaileddiff()
       return 1
@@ -543,17 +552,12 @@ function runcheck(name, hide)
   end
   local errorlevel = 0
   for _,engine in pairs(checkengines) do
-    local enginename = engine
-    -- Allow for luatex == luajittex for .tlg/.tpf purposes
-    if engine == "luajittex" then
-      engine = "luatex"
-    end
     setup_check(name,engine)
     local errlevel = 0
     if fileexists(testfiledir .. "/" .. name .. pvtext) then
-      errlevel = check_and_diff(pvtext,engine,enginename,compare_pdf,true)
+      errlevel = check_and_diff(pvtext,engine,compare_pdf,true)
     else
-      errlevel = check_and_diff(lvtext,engine,enginename,compare_tlg)
+      errlevel = check_and_diff(lvtext,engine,compare_tlg)
     end
     if errlevel ~= 0 and options["halt-on-error"] then
       return 1
@@ -617,7 +621,7 @@ function compare_pdf(name,engine,cleanup)
   return errorlevel
 end
 
-function compare_tlg(name, engine,cleanup)
+function compare_tlg(name,engine,cleanup)
   local errorlevel
   local testname = name .. "." .. engine
   local difffile = testdir .. "/" .. testname .. os_diffext
@@ -628,10 +632,9 @@ function compare_tlg(name, engine,cleanup)
   end
   -- Do additional log formatting if the engine is LuaTeX, there is no
   -- LuaTeX-specific .tlg file and the default engine is not LuaTeX
-  if engine == "luatex"
+  if match(engine,"^lua")
     and not match(tlgfile, "%.luatex" .. "%" .. tlgext)
-    and stdengine ~= "luatex"
-    and stdengine ~= "luajittex"
+    and not match(stdengine,"^lua")
     then
     local lualogfile = logfile
     if cleanup then
@@ -662,29 +665,25 @@ function runtest(name, engine, hide, ext, pdfmode, breakout)
   local lvtfile = name .. (ext or lvtext)
   cp(lvtfile, fileexists(testfiledir .. "/" .. lvtfile)
     and testfiledir or unpackdir, testdir)
+  local checkopts = checkopts
   local engine = engine or stdengine
-  -- Set up the format file name if it's one ending "...tex"
-  local realengine = engine
-  local format = ""
-  if
-    match(checkformat, "tex$") and
-    not match(engine, checkformat) then
-    format = " -fmt=" .. gsub(engine, "(.*)tex$", "%1") .. checkformat
+  local binary = engine
+  local format = gsub(engine,"tex$",checkformat)
+  -- Special binary/format combos
+  if specialformats[checkformat] and next(specialformats[checkformat]) then
+    local t = specialformats[checkformat]
+    if t[engine] and next(t[engine]) then
+      local t = t[engine]
+      binary    = t.binary  or binary
+      checkopts = t.options or checkopts
+      format    = t.format  or format
+    end
   end
-  -- Special casing for e-LaTeX format
-  if
-    match(checkformat, "^latex$") and
-    match(engine, "^etex$") then
-    format = " -fmt=latex"
-  end
-  -- Special casing for (u)pTeX LaTeX formats
-  if
-    match(checkformat, "^latex$") and
-    match(engine, "^u?ptex$") then
-    realengine = "e" .. engine
+  -- Finalise format string
+  if format ~= "" then
+    format = " --fmt=" .. format
   end
   -- Special casing for XeTeX engine
-  local checkopts = checkopts
   if match(engine, "xetex") and not pdfmode then
     checkopts = checkopts .. " -no-pdf"
   end
@@ -692,19 +691,8 @@ function runtest(name, engine, hide, ext, pdfmode, breakout)
   local function setup(file)
     return " -jobname=" .. name .. " " .. ' "\\input ' .. file .. '" '
   end
-  if match(checkformat, "^context$") then
-    format = ""
+  if match(checkformat,"^context$") then
     function setup(file) return ' "' .. file .. '" '  end
-    if engine == "luatex" or engine == "luajittex" then
-      realengine = "context"
-    elseif engine == "pdftex" then
-      realengine = "texexec"
-    elseif engine == "xetex" then
-      realengine = "texexec --xetex"
-    else
-      print("Engine incompatible with format")
-      exit(1)
-    end
   end
   local basename = testdir .. "/" .. name
   local logfile = basename .. logext
@@ -713,7 +701,7 @@ function runtest(name, engine, hide, ext, pdfmode, breakout)
   local npffile = basename .. "." .. engine .. pdfext
   local asciiopt = ""
   for _,i in ipairs(asciiengines) do
-    if realengine == i then
+    if binary == i then
       asciiopt = "-translate-file ./ascii.tcx "
       break
     end
@@ -733,11 +721,14 @@ function runtest(name, engine, hide, ext, pdfmode, breakout)
       -- Avoid spurious output from (u)pTeX
       os_setenv .. " GUESS_INPUT_KANJI_ENCODING=0"
         .. os_concat ..
+      -- Allow for local texmf files
+      os_setenv .. " TEXMFCNF=." .. os_pathsep
+        .. os_concat ..
       (forcecheckepoch and setepoch() or "") ..
       -- Ensure lines are of a known length
       os_setenv .. " max_print_line=" .. maxprintline
         .. os_concat ..
-      realengine .. format 
+      binary .. format 
         .. " " .. asciiopt .. " " .. checkopts
         .. setup(lvtfile)
         .. (hide and (" > " .. os_null) or "")
